import json
import traceback

from tornado.web import RequestHandler
from agent.domain import DOMAINOBJ, loadDoamin, DomainNotExist
from agent.common import pylog

from tornado.httpclient import HTTPClient, HTTPRequest, HTTPError
from tornado.concurrent import run_on_executor
from concurrent.futures import ThreadPoolExecutor
from tornado.gen import coroutine

class ConfigureHandler(RequestHandler):
    executor = ThreadPoolExecutor(20)

    @run_on_executor
    def _response(self,
                  response_data:dict,
                  response_ip  :str,
                  response_port:str):
        http_client = HTTPClient()
        try:
            URL = "http://{ip}:{port}/apply_result".format(
                            ip = response_ip, port = response_port)
            pylog.logger.info("response to {}".format(URL))

            response = http_client.fetch(HTTPRequest(
                url    =  URL,
                method = "POST",
                body   = json.dumps(response_data)
            ))
        except RuntimeError as e:
            pylog.logger.error("Failed to response to {}: {}".format(URL, e))
            return False, "{},{}".format(e, traceback.format_exc())

        except HTTPError as e:
            pylog.logger.error("Failed to response to {}: {}".format(URL, e))
            return False, "{},{}".format(e, traceback.format_exc())

        except Exception as e:
            pylog.logger.error("Failed to response to {}: {}".format(URL, e))
            return False, "{},{}".format(e, traceback.format_exc())

        else:
            if response.code == 200:
                return True, ""
            else:
                return False, response.reason

        finally:
            http_client.close()


    @run_on_executor
    def _configureImpl(self, param_domain_dict:dict, readonly:bool):
        """ Call getParamAll() function or setParamAll() function of each parameter domain in turn
        """
        global DOMAINOBJ
        domain_result = {}
        for domain in param_domain_dict.keys():
            try:
                loadDoamin(domain)
                param_list = param_domain_dict[domain]
                if readonly:
                    _, out = DOMAINOBJ[domain].getParamAll(param_list)
                else:
                    _, out = DOMAINOBJ[domain].setParamAll(param_list)
                domain_result[domain] = out
                
            except DomainNotExist as e:
                domain_result[domain] = "{}".format(e)
            
            except Exception as e:
                domain_result[domain] = "{}".format(e)
                
        return domain_result


    @coroutine
    def post(self):
        global DOMAINOBJ
        def _validField(request_data):
            for _key in ['readonly', 'resp_ip', 'resp_port', 'target_id', 'data']:
                if not request_data.__contains__(_key):
                    return False
            return True

        request_data = json.loads(self.request.body)
        if not _validField(request_data):
            pylog.logger.error("invalid request data: {}".format(request_data))
            self.write(json.dumps({"suc" : False, "msg": "invalid request data: {}".format(request_data)}))
            self.finish()
            
        else:
            pylog.logger.info("get configure request: {}".format(request_data))
            self.write(json.dumps({"suc" : True, "msg" : "parameter set/get is running"}))
            self.finish()

            out = yield self._configureImpl(request_data['data'], request_data['readonly'])
            pylog.logger.info("configure result: {}".format(out))
            
            response_data = {
                "suc"       : True, 
                "data"      : out, 
                "target_id" : request_data['target_id'], 
                "msg"       : ""
            }
            
            # if suc:
            #     response_data = {
            #         "suc"       : True, 
            #         "data"      : out, 
            #         "target_id" : request_data['target_id'], 
            #         "msg"       : ""
            #     }
            # else:     
            #     response_data = {
            #         "suc"       : False, 
            #         "data"      : {}, 
            #         "target_id" : request_data['target_id'], 
            #         "msg"       : out
            #     }   
            _, _ = yield self._response(response_data, request_data['resp_ip'], request_data['resp_port'])