import os
from importlib import import_module

from agent import domain
from agent.common import pylog
from agent.common.config import Config
from agent.common.system import appendDomain
from agent.common.system import getAllDomain


class DomainNotExist(Exception):
    pass

DOMAINOBJ = {}
INITIAL_DOMAIN_LIST = ['sysctl', 'nginx_conf', 'net']


def loadDoamin(domain_name):
    global DOMAINOBJ
    if DOMAINOBJ.__contains__(domain_name):
        return
    
    try:
        module = import_module('agent.domain.{}'.format(domain_name))
        domain_obj = eval('domain.{domain_name}.{class_name}()'.format(
            domain_name = domain_name,
            class_name = "".join([value.capitalize() for value in domain_name.split("_")]) if "_" in domain_name else domain_name.capitalize()
        ))

    except Exception as e:
        pylog.logger.warning("parameter domain {} is not supported by current environment: {}".format(domain_name,e))
        raise DomainNotExist("parameter domain {} is not supported by current environment: {}".format(domain_name,e))

    else:
        pylog.logger.info("Load parameter domain '{}' ({})".format(domain_name, module))
        DOMAINOBJ[domain_name] = domain_obj
        appendDomain(domain_name)


def loadSupportedDoamin():
    """ try to load all avaliable domain in folder /agent/domain

    """
    all_domain_list = getAllDomain()
    for domain_name in all_domain_list:
        try:
            loadDoamin(domain_name)
            
        except DomainNotExist as e:
            continue
        
        except Exception as e:
            continue


def initialLoadDomain():
    """ try to init domain in file /etc/keentune/agent/domain.conf

    """
    file_path = os.path.join(Config.DOMAIN_CONF, "domain.conf")
    if not os.path.exists(Config.DOMAIN_CONF):
        os.makedirs(Config.DOMAIN_CONF)
    if not os.path.exists(file_path):
        with open(file_path, 'w') as f:
            data = [domain+"\n" for domain in INITIAL_DOMAIN_LIST]
            f.writelines(data)
    loadSupportedDoamin()
