import os
import time
from collections import defaultdict

from agent.common.config import Config
from agent.common.pylog import functionLog
from agent.common.system import sysCommand

PATH_MYCNF  = "/etc/my.cnf"
GET_VALUE   = "cat /etc/my.cnf |grep -w {name} | tail -n1 | awk -F'=' '{{print$2}}'"
SET_VALUE   = "echo {name}={value} >> /etc/my.cnf"
GET_FLAG    = "cat /etc/my.cnf  |grep -w {name} | tail -n1 |wc -L |awk '{{if (sum=$(wc -L \"{name}\") && $0==$sum) print \"SET_TRUE\";else print \"SET_FALSE\"}}'"
SET_FLAG    = "echo {name} >> /etc/my.cnf"
MYCNF_DATA  = "[client-server]\n!includedir /etc/my.cnf.d\n[mysqld]\n"

class MyCnf:
    def __init__(self):
        super().__init__()
        if not os.path.exists(PATH_MYCNF):
            raise Exception("Can not find config file: file /etc/my.cnf do not exists!")
        
        self.backup_file = os.path.join(Config.BACKUP_PATH, "my.cnf")


    @functionLog
    def setParamAll(self, param_list: dict):
        with open(PATH_MYCNF,"w") as f:
            f.write(MYCNF_DATA)

        result = {}
        for param_name, param_info in param_list.items():
            if param_info['value'] == "":
                continue

            if param_name in ["core-file", "skip_ssl", "skip_name_resolve"]:
                if param_info["value"] == "SET_TRUE":
                    suc, res = sysCommand(SET_FLAG.format(name = param_name.strip()))
            else:
                suc, res = sysCommand(SET_VALUE.format(name=param_name.strip(),value=param_info['value']))

            result[param_name] = {
                "value": param_info["value"],
                "dtype": param_info["dtype"],
                "suc": suc,
                "msg": res
            }

        return True, result


    @functionLog
    def getParamAll(self, param_list: dict):
        result = {}
        SUC = True
        for param_name, param_info in param_list.items():
            if param_name in ["core-file", "skip_ssl", "skip_name_resolve"]:
                suc, res = sysCommand(GET_FLAG.format(name = param_name.strip()))
            else:
                suc, res = sysCommand(GET_VALUE.format(name=param_name.strip()))

            if suc:
                result[param_name] = {
                    "value": res,
                    "dtype": param_info["dtype"],
                    "suc": True,
                    "msg": ""
                }
            else:
                SUC = False
                result[param_name] = {
                    "value": "",
                    "dtype": param_info["dtype"],
                    "suc": False,
                    "msg": res
                }
        return SUC, result


    @functionLog
    def rollback(self):
        if not os.path.exists(self.backup_file):
            return True, "Can not find backup file:{}".format(self.backup_file)

        suc, res = sysCommand("echo y | cp {} {}".format(self.backup_file, PATH_MYCNF))
        if suc:
            backup_time = time.asctime(time.localtime(os.path.getctime(self.backup_file)))
            os.remove(self.backup_file)
            return True, backup_time
        else:
            return False, res


    @functionLog
    def backup(self, param_list: dict):
        suc, _ = self.rollback()
        if not suc:
            return None

        suc, _ = sysCommand("echo y | cp {} {}".format(PATH_MYCNF, self.backup_file))
        if not suc:
            return None
        
        avaliable_knobs = defaultdict(dict)
        for param_name, _ in param_list.items():
            avaliable_knobs[param_name]['avaliable'] = True
            avaliable_knobs[param_name]['value']     = None
            avaliable_knobs[param_name]['msg']       = ""
        return avaliable_knobs